@@ -13,6 +13,7 @@ Last.fmの仕様で現在再生中の曲について、再生開始時刻が取
 
 # python
 import urllib
+import urllib2
 import json
 import datetime
 import time
@@ -20,6 +21,13 @@ import threading
 
 import pytz
 
+class NowfmError(Exception):
+    def __init__(self, value, detail=""):
+        self.value = value
+        self.detail = detail
+
+    def __str__(self):
+        return repr(self.value)
 
 class Nowfm(object):
     endpoint = 'http://ws.audioscrobbler.com/2.0/'
@@ -30,24 +38,24 @@ class Nowfm(object):
             Nowfm.endpoint, api_key)
 
     def get_nowplaying(self, user):
-        try:
-            return self._get_nowplaying(user)
-        except Exception:
-            return None
-
-    def _get_nowplaying(self, user):
         """
         ユーザーの現在再生中の曲を再生開始時刻情報を付加して返す
         """
         method = 'user.getrecenttracks'
         url = self.url_template + '&method={0}&limit=1&user={1}'.format(
             method, user)
-        jobj = json.loads(unicode(urllib.urlopen(url).read(), 'utf-8'))
+        src = self._get_src(url)
+        jobj = json.loads(src)
         tracks = jobj['recenttracks']['track']
-        now = tracks[0]
-
+        if type(tracks) is not list:
+            raise NowfmError("{0} is not currently playing track".format(user),
+                             'type(tracks) is not list')
+        if len(tracks) < 2:
+            raise NowfmError("{0} is not currently playing track".format(user),
+                             "len(tracks) < 2")
+        now, prev = tracks[0], tracks[1]
         ldate = []
-        t1 = threading.Thread(target=self._thread_calc_date, args=(tracks[1], ldate))
+        t1 = threading.Thread(target=self._thread_calc_date, args=(prev, ldate))
         t1.start()
 
         linfo = []
@@ -58,10 +66,22 @@ class Nowfm(object):
 
         info = linfo[0]
         date = ldate[0]
-        info['@attr'] = now['@attr']
+        if '@attr' in now:
+            info['@attr'] = now['@attr']
         info['@date'] = date
         return info
 
+    def _get_src(self, url):
+        opener = urllib2.build_opener()
+        opener.addheaders = [('User-agent', 'Mozilla/5.0')]
+        f = opener.open(url)
+        if f.getcode() == 404:
+            raise NowfmError("404 Not Found")
+        if f.info().gettype() == 'text/xml':
+            raise NowfmError("503 Service Unavailable",
+                             "Last.fm Server returns text/xml")
+        return unicode(f.read(), 'utf-8')
+
     def _calc_date(self, prev):
         """
         現在再生中の曲の再生開始時刻を計算
@@ -80,7 +100,8 @@ class Nowfm(object):
         # 1つ前の曲の再生開始時刻のタイムスタンプ
         prev_start = int(prev['date']['uts'])
         # 1つ前の曲の演奏時間
-        prev_duration = int(self._get_track_info(prev)['duration']) / 1000
+        prev_info = self._get_track_info(prev)
+        prev_duration = int(prev_info['duration']) / 1000
         # 1つ前の曲の再生終了時刻のタイムスタンプ
         prev_end = prev_start + prev_duration
         # 現在の曲の再生開始時刻のタイムスタンプ
@@ -114,7 +135,8 @@ class Nowfm(object):
         else:
             url += '&track={0}&artist={1}'.format(
                 track['name'], track['artist']['#text'])
-        jobj = json.loads(unicode(urllib.urlopen(url).read(), 'utf-8'))
+        src = self._get_src(url)
+        jobj = json.loads(src)
         return jobj['track']
 
     def _thread_get_track_info(self, track, ret):
